@@ -36,3 +36,17 @@ END_ICON = Icon(
     icon_anchor=[12, 41],
     popup_anchor=[1, -34],
     shadow_size=[41, 41])
+
+# Options that can be altered by user
+options = {
+    "OSM_TIMEOUT": 180,
+    "OSM_RETRIES": 3,
+    "OSM_BOUNDING_BOX_OPTIMIZATION": True,
+    "OSM_BOUNDING_BOX_SPLIT_IOU_THRES": .5,
+    "OSM_SINGLE_BOUNDING_BOX": False,
+    "SOCKET_TIMEOUT": 300,
+    "MAP_MATCHING_DEFAULT_ALGORITHM": "nx",
+    "MAP_MATCHING_V_THRES": 1.0,
+    "MAP_MATCHING_ALPHA": 1.0,
+    "MAP_MATCHING_BETA": 1.0
+}
