@@ -86,6 +86,11 @@ class RDYFile:
         self.ridy_version: Optional[str] = None
         self.ridy_version_code: Optional[int] = None
 
+        # Geographic extent of file (can only be determined if GPSSeries was recorded
+        self.lat_sw, self.lon_sw = None, None
+        self.lat_ne, self.lon_ne = None, None
+        self.bbox = None
+
         # RDY File Infos
         self.rdy_format_version: Optional[float] = None
         self.rdy_info_name: Optional[str] = None
@@ -146,6 +151,15 @@ class RDYFile:
 
             if self.sync_method:
                 self._synchronize()
+
+            if len(self.measurements[GPSSeries]) > 0:
+                self.lon_sw = self.measurements[GPSSeries].lon.min()
+                self.lon_ne = self.measurements[GPSSeries].lon.max()
+
+                self.lat_sw = self.measurements[GPSSeries].lat.min()
+                self.lat_ne = self.measurements[GPSSeries].lat.max()
+
+                self.bbox = [self.lon_sw, self.lat_sw, self.lon_ne, self.lat_ne]
         else:
             logging.warning("RDYFile instantiated without a path")
 
@@ -282,7 +296,8 @@ class RDYFile:
                 for m in self.measurements.values():
                     m.synchronize("gps_time", sync_timestamp, sync_time, timedelta_unit=self.timedelta_unit)
             else:
-                logger.warning("(%s) No GPS time recording, falling back to device_time synchronization" % self.filename)
+                logger.warning(
+                    "(%s) No GPS time recording, falling back to device_time synchronization" % self.filename)
                 self.sync_method = "device_time"
                 self._synchronize()
         elif self.sync_method == "ntp_time":
@@ -419,7 +434,10 @@ class RDYFile:
 
             return center_lon, center_lat
 
-    def do_map_matching(self, v_thres: float = 1, algorithm: str = "pyridy", alpha: int = 1.0, beta: int = 1.0):
+    def do_map_matching(self, v_thres: float = config.options["MAP_MATCHING_V_THRES"],
+                        algorithm: str = config.options["MAP_MATCHING_DEFAULT_ALGORITHM"],
+                        alpha: int = config.options["MAP_MATCHING_ALPHA"],
+                        beta: int = config.options["MAP_MATCHING_BETA"]):
         """ Performs map matching of the GPS track to closest OSM nodes/ways
 
         Parameters
@@ -429,10 +447,10 @@ class RDYFile:
             Graph
         beta: float, default: 1.0
             Beta is a scaling factor for the emission probabilities.
-        algorithm: str, default: pyridy
+        algorithm: str, default: nx
             Algorithm to be used, can be "pyridy" or "nx". The pyridy algorithm also incorporates how switches
             can be transited
-        v_thres: float
+        v_thres: float, default: 1.0
             Speed threshold, GPS points measured with a velocity below v_thres [m/s] will not be considered
 
         Returns
@@ -460,10 +478,10 @@ class RDYFile:
 
             # Initialize edge weights
             for e in self.osm.G.edges:
-                self.osm.G.edges[e]["c_weight"] = 1*alpha
+                self.osm.G.edges[e]["c_weight"] = 1 * alpha
 
             for k in c_edges.keys():
-                self.osm.G.edges[k]["c_weight"] = 1 / (1 + beta*sum(c_edges[k]["e_prob"]))
+                self.osm.G.edges[k]["c_weight"] = 1 / (1 + beta * sum(c_edges[k]["e_prob"]))
 
             # Perform map matching
             s_n = None
@@ -493,8 +511,8 @@ class RDYFile:
 
                 m_w_ids = [self.osm.G[n1][n2][0]["way_id"] for n1, n2 in zip(m_n_ids, m_n_ids[1:])]
                 self.matched_ways = list(set([self.osm.way_dict[w_id] for w_id in m_w_ids]))  # Mapped Ways
-                logger.info("(%s) Found %d nodes that match the files GPS track!" % (self.filename,
-                                                                                     len(self.matched_nodes)))
+                logger.debug("(%s) Found %d nodes that match the files GPS track!" % (self.filename,
+                                                                                      len(self.matched_nodes)))
         else:
             logger.warning("(%s) Can't do map matching since no file contains no OSM data" % self.filename)
         pass
@@ -544,7 +562,7 @@ class RDYFile:
         path Path to the ridy file
 
         """
-        logger.info("Loading file: %s" % path)
+        logger.debug("Loading file: %s" % path)
 
         _, self.extension = os.path.splitext(path)
         _, self.filename = os.path.split(path)
@@ -556,49 +574,49 @@ class RDYFile:
             if 'Ridy_Version' in rdy:
                 self.ridy_version = rdy['Ridy_Version']
             else:
-                logger.info("No Ridy_Version in file: %s" % self.filename)
+                logger.debug("No Ridy_Version in file: %s" % self.filename)
                 self.ridy_version = None
 
             if 'Ridy_Version_Code' in rdy:
                 self.ridy_version_code = rdy['Ridy_Version_Code']
             else:
-                logger.info("No Ridy_Version_Code in file: %s" % self.filename)
+                logger.debug("No Ridy_Version_Code in file: %s" % self.filename)
                 self.ridy_version_code = None
 
             if 'RDY_Format_Version' in rdy:
                 self.rdy_format_version = rdy['RDY_Format_Version']
             else:
-                logger.info("No RDY_Format_Version in file: %s" % self.filename)
+                logger.debug("No RDY_Format_Version in file: %s" % self.filename)
                 self.rdy_format_version = None
 
             if 'RDY_Info_Name' in rdy:
                 self.rdy_info_name = rdy['RDY_Info_Name']
             else:
-                logger.info("No RDY_Info_Name in file: %s" % self.filename)
+                logger.debug("No RDY_Info_Name in file: %s" % self.filename)
                 self.rdy_info_name = None
 
             if 'RDY_Info_Sex' in rdy:
                 self.rdy_info_sex = rdy['RDY_Info_Sex']
             else:
-                logger.info("No RDY_Info_Sex in file: %s" % self.filename)
+                logger.debug("No RDY_Info_Sex in file: %s" % self.filename)
                 self.rdy_info_sex = None
 
             if 'RDY_Info_Age' in rdy:
                 self.rdy_info_age = rdy['RDY_Info_Age']
             else:
-                logger.info("No RDY_Info_Age in file: %s" % self.filename)
+                logger.debug("No RDY_Info_Age in file: %s" % self.filename)
                 self.rdy_info_age = None
 
             if 'RDY_Info_Height' in rdy:
                 self.rdy_info_height = rdy['RDY_Info_Height']
             else:
-                logger.info("No RDY_Info_Height in file: %s" % self.filename)
+                logger.debug("No RDY_Info_Height in file: %s" % self.filename)
                 self.rdy_info_height = None
 
             if 'RDY_Info_Weight' in rdy:
                 self.rdy_info_weight = rdy['RDY_Info_Weight']
             else:
-                logger.info("No RDY_Info_Weight in file: %s" % self.filename)
+                logger.debug("No RDY_Info_Weight in file: %s" % self.filename)
                 self.rdy_info_weight = None
 
             if 't0' in rdy:
@@ -609,31 +627,31 @@ class RDYFile:
                     self.t0 = np.datetime64(rdy['t0'])
             else:
                 self.t0 = None
-                logger.info("No t0 in file: %s" % self.filename)
+                logger.debug("No t0 in file: %s" % self.filename)
 
             if 'cs_matrix_string' in rdy:
                 self.cs_matrix_string = rdy['cs_matrix_string']
             else:
                 self.cs_matrix_string = None
-                logger.info("No t0 in file: %s" % self.filename)
+                logger.debug("No t0 in file: %s" % self.filename)
 
             if 'timestamp_when_started' in rdy:
                 self.timestamp_when_started = rdy['timestamp_when_started']
             else:
                 self.timestamp_when_started = None
-                logger.info("No timestamp_when_started in file: %s" % self.filename)
+                logger.debug("No timestamp_when_started in file: %s" % self.filename)
 
             if 'timestamp_when_stopped' in rdy:
                 self.timestamp_when_stopped = rdy['timestamp_when_stopped']
             else:
                 self.timestamp_when_stopped = None
-                logger.info("No timestamp_when_stopped in file: %s" % self.filename)
+                logger.debug("No timestamp_when_stopped in file: %s" % self.filename)
 
             if 'ntp_timestamp' in rdy:
                 self.ntp_timestamp = rdy['ntp_timestamp']
             else:
                 self.ntp_timestamp = None
-                logger.info("No ntp_timestamp in file: %s" % self.filename)
+                logger.debug("No ntp_timestamp in file: %s" % self.filename)
 
             if 'ntp_date_time' in rdy:
                 if self.strip_timezone:
@@ -647,18 +665,18 @@ class RDYFile:
                     self.ntp_date_time = np.datetime64(rdy['t0'])
             else:
                 self.ntp_date_time = None
-                logger.info("No ntp_date_time in file: %s" % self.filename)
+                logger.debug("No ntp_date_time in file: %s" % self.filename)
 
             if "device" in rdy:
                 self.device = Device(**rdy['device_info'])
             else:
-                logger.info("No device information in file: %s" % self.filename)
+                logger.debug("No device information in file: %s" % self.filename)
 
             if "sensors" in rdy:
                 for sensor in rdy['sensors']:
                     self.sensors.append(Sensor(**sensor))
             else:
-                logger.info("No sensor descriptions in file: %s" % self.filename)
+                logger.debug("No sensor descriptions in file: %s" % self.filename)
 
             if (self._series is not None and AccelerationSeries in self._series) or self._series is None:
                 if "acc_series" in rdy:
@@ -667,14 +685,14 @@ class RDYFile:
                                            rdy_format_version=self.rdy_format_version,
                                            **rdy['acc_series'])
                 else:
-                    logger.info("No Acceleration Series in file: %s" % self.filename)
+                    logger.debug("No Acceleration Series in file: %s" % self.filename)
 
             if (self._series is not None and AccelerationUncalibratedSeries in self._series) or self._series is None:
                 if "acc_uncal_series" in rdy:
                     self.measurements[AccelerationUncalibratedSeries] = AccelerationUncalibratedSeries(
-                        filename=self.filename,rdy_format_version=self.rdy_format_version, **rdy['acc_uncal_series'])
+                        filename=self.filename, rdy_format_version=self.rdy_format_version, **rdy['acc_uncal_series'])
                 else:
-                    logger.info("No uncalibrated Acceleration Series in file: %s" % self.filename)
+                    logger.debug("No uncalibrated Acceleration Series in file: %s" % self.filename)
 
             if (self._series is not None and LinearAccelerationSeries in self._series) or self._series is None:
                 if "lin_acc_series" in rdy:
@@ -683,7 +701,7 @@ class RDYFile:
                         rdy_format_version=self.rdy_format_version,
                         **rdy['lin_acc_series'])
                 else:
-                    logger.info("No Linear Acceleration Series in file: %s" % self.filename)
+                    logger.debug("No Linear Acceleration Series in file: %s" % self.filename)
 
             if (self._series is not None and MagnetometerSeries in self._series) or self._series is None:
                 if "mag_series" in rdy:
@@ -692,14 +710,14 @@ class RDYFile:
                                            rdy_format_version=self.rdy_format_version,
                                            **rdy['mag_series'])
                 else:
-                    logger.info("No Magnetometer Series in file: %s" % self.filename)
+                    logger.debug("No Magnetometer Series in file: %s" % self.filename)
 
             if (self._series is not None and MagnetometerUncalibratedSeries in self._series) or self._series is None:
                 if "mag_uncal_series" in rdy:
                     self.measurements[MagnetometerUncalibratedSeries] = MagnetometerUncalibratedSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **rdy['mag_uncal_series'])
                 else:
-                    logger.info("No uncalibrated Magnetometer Series in file: %s" % self.filename)
+                    logger.debug("No uncalibrated Magnetometer Series in file: %s" % self.filename)
 
             if (self._series is not None and OrientationSeries in self._series) or self._series is None:
                 if "orient_series" in rdy:
@@ -707,7 +725,7 @@ class RDYFile:
                                                                              rdy_format_version=self.rdy_format_version,
                                                                              **rdy['orient_series'])
                 else:
-                    logger.info("No Orientation Series in file: %s" % self.filename)
+                    logger.debug("No Orientation Series in file: %s" % self.filename)
 
             if (self._series is not None and GyroSeries in self._series) or self._series is None:
                 if "gyro_series" in rdy:
@@ -715,14 +733,14 @@ class RDYFile:
                                                                rdy_format_version=self.rdy_format_version,
                                                                **rdy['gyro_series'])
                 else:
-                    logger.info("No Gyro Series in file: %s" % self.filename)
+                    logger.debug("No Gyro Series in file: %s" % self.filename)
 
             if (self._series is not None and GyroUncalibratedSeries in self._series) or self._series is None:
                 if "gyro_uncal_series" in rdy:
                     self.measurements[GyroUncalibratedSeries] = GyroUncalibratedSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **rdy['gyro_uncal_series'])
                 else:
-                    logger.info("No uncalibrated Gyro Series in file: %s" % self.filename)
+                    logger.debug("No uncalibrated Gyro Series in file: %s" % self.filename)
 
             if (self._series is not None and RotationSeries in self._series) or self._series is None:
                 if "rot_series" in rdy:
@@ -730,7 +748,7 @@ class RDYFile:
                                                                        rdy_format_version=self.rdy_format_version,
                                                                        **rdy['rot_series'])
                 else:
-                    logger.info("No Rotation Series in file: %s" % self.filename)
+                    logger.debug("No Rotation Series in file: %s" % self.filename)
 
             if (self._series is not None and GPSSeries in self._series) or self._series is None:
                 if "gps_series" in rdy:
@@ -738,28 +756,28 @@ class RDYFile:
                                                              rdy_format_version=self.rdy_format_version,
                                                              **rdy['gps_series'])
                 else:
-                    logger.info("No GPS Series in file: %s" % self.filename)
+                    logger.debug("No GPS Series in file: %s" % self.filename)
 
             if (self._series is not None and GNSSMeasurementSeries in self._series) or self._series is None:
                 if "gnss_series" in rdy:
                     self.measurements[GNSSMeasurementSeries] = GNSSMeasurementSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **rdy['gnss_series'])
                 else:
-                    logger.info("No GPS Series in file: %s" % self.filename)
+                    logger.debug("No GPS Series in file: %s" % self.filename)
 
             if (self._series is not None and GNSSClockMeasurementSeries in self._series) or self._series is None:
                 if "gnss_clock_series" in rdy:
                     self.measurements[GNSSClockMeasurementSeries] = GNSSClockMeasurementSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **rdy['gnss_clock_series'])
                 else:
-                    logger.info("No GNSS Clock Series in file: %s" % self.filename)
+                    logger.debug("No GNSS Clock Series in file: %s" % self.filename)
 
             if (self._series is not None and NMEAMessageSeries in self._series) or self._series is None:
                 if "nmea_series" in rdy:
                     self.measurements[NMEAMessageSeries] = NMEAMessageSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **rdy['nmea_series'])
                 else:
-                    logger.info("No GPS Series in file: %s" % self.filename)
+                    logger.debug("No GPS Series in file: %s" % self.filename)
 
             if (self._series is not None and PressureSeries in self._series) or self._series is None:
                 if "pressure_series" in rdy:
@@ -767,7 +785,7 @@ class RDYFile:
                                                                        rdy_format_version=self.rdy_format_version,
                                                                        **rdy['pressure_series'])
                 else:
-                    logger.info("No Pressure Series in file: %s" % self.filename)
+                    logger.debug("No Pressure Series in file: %s" % self.filename)
 
             if (self._series is not None and TemperatureSeries in self._series) or self._series is None:
                 if "temperature_series" in rdy:
@@ -775,7 +793,7 @@ class RDYFile:
                                                                              rdy_format_version=self.rdy_format_version,
                                                                              **rdy['temperature_series'])
                 else:
-                    logger.info("No Temperature Series in file: %s" % self.filename)
+                    logger.debug("No Temperature Series in file: %s" % self.filename)
 
             if (self._series is not None and HumiditySeries in self._series) or self._series is None:
                 if "humidity_series" in rdy:
@@ -783,7 +801,7 @@ class RDYFile:
                                                                        rdy_format_version=self.rdy_format_version,
                                                                        **rdy['humidity_series'])
                 else:
-                    logger.info("No Humidity Series in file: %s" % self.filename)
+                    logger.debug("No Humidity Series in file: %s" % self.filename)
 
             if (self._series is not None and LightSeries in self._series) or self._series is None:
                 if "light_series" in rdy:
@@ -791,7 +809,7 @@ class RDYFile:
                                                                  rdy_format_version=self.rdy_format_version,
                                                                  **rdy['light_series'])
                 else:
-                    logger.info("No Light Series in file: %s" % self.filename)
+                    logger.debug("No Light Series in file: %s" % self.filename)
 
             if (self._series is not None and WzSeries in self._series) or self._series is None:
                 if "wz_series" in rdy:
@@ -799,7 +817,7 @@ class RDYFile:
                                                            rdy_format_version=self.rdy_format_version,
                                                            **rdy['wz_series'])
                 else:
-                    logger.info("No Wz Series in file: %s" % self.filename)
+                    logger.debug("No Wz Series in file: %s" % self.filename)
 
             if (self._series is not None and SubjectiveComfortSeries in self._series) or self._series is None:
                 if "subjective_comfort_series" in rdy:
@@ -808,7 +826,7 @@ class RDYFile:
                         rdy_format_version=self.rdy_format_version,
                         **rdy['subjective_comfort_series'])
                 else:
-                    logger.info("No Subjective Comfort Series in file: %s" % self.filename)
+                    logger.debug("No Subjective Comfort Series in file: %s" % self.filename)
                 pass
 
         elif self.extension == ".sqlite":
@@ -821,6 +839,7 @@ class RDYFile:
                 # Older files can contain wrong table name
                 try:
                     info = dict(pd.read_sql_query("SELECT * from measurment_information_table", db_con))
+                    logger.debug("(%s) Older file containing measu(rm)ent_information_table" % self.filename)
                 except (DatabaseError, PandasDatabaseError) as e:
                     logger.error(
                         "(%s) DatabaseError occurred when accessing measurement_information_table" % self.filename)
@@ -846,7 +865,7 @@ class RDYFile:
 
             # Info
             if 'ridy_version' in info and len(info['ridy_version']) > 1:
-                logger.info("Measurement information table contains more than 1 row!")
+                logger.debug("Measurement information table contains more than 1 row!")
 
             if 'ridy_version' in info and len(info['ridy_version']) > 0:
                 self.ridy_version = info['ridy_version'].iloc[-1]
@@ -920,7 +939,8 @@ class RDYFile:
                     self.measurements[AccelerationUncalibratedSeries] = AccelerationUncalibratedSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **dict(acc_uncal_df))
                 except (DatabaseError, PandasDatabaseError) as e:
-                    logger.error("(%s) DatabaseError occurred when accessing acc_uncal_measurements_table" % self.filename)
+                    logger.error(
+                        "(%s) DatabaseError occurred when accessing acc_uncal_measurements_table" % self.filename)
                     logger.error(e)
 
             if (self._series is not None and LinearAccelerationSeries in self._series) or self._series is None:
@@ -951,7 +971,8 @@ class RDYFile:
                     self.measurements[MagnetometerUncalibratedSeries] = MagnetometerUncalibratedSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **dict(mag_uncal_df))
                 except (DatabaseError, PandasDatabaseError) as e:
-                    logger.error("(%s) DatabaseError occurred when accessing mag_uncal_measurements_table" % self.filename)
+                    logger.error(
+                        "(%s) DatabaseError occurred when accessing mag_uncal_measurements_table" % self.filename)
                     logger.error(e)
 
             if (self._series is not None and OrientationSeries in self._series) or self._series is None:
@@ -1020,7 +1041,8 @@ class RDYFile:
                     self.measurements[GNSSClockMeasurementSeries] = GNSSClockMeasurementSeries(
                         filename=self.filename, rdy_format_version=self.rdy_format_version, **dict(gnss_clock_df))
                 except (DatabaseError, PandasDatabaseError) as e:
-                    logger.error("(%s) DatabaseError occurred when accessing gnss_clock_measurement_table" % self.filename)
+                    logger.error(
+                        "(%s) DatabaseError occurred when accessing gnss_clock_measurement_table" % self.filename)
                     logger.error(e)
 
             if (self._series is not None and NMEAMessageSeries in self._series) or self._series is None:
@@ -1087,7 +1109,8 @@ class RDYFile:
 
             if (self._series is not None and SubjectiveComfortSeries in self._series) or self._series is None:
                 try:
-                    subjective_comfort_df = pd.read_sql_query("SELECT * from subjective_comfort_measurements_table", db_con)
+                    subjective_comfort_df = pd.read_sql_query("SELECT * from subjective_comfort_measurements_table",
+                                                              db_con)
                     self.measurements[SubjectiveComfortSeries] = SubjectiveComfortSeries(
                         filename=self.filename,
                         rdy_format_version=self.rdy_format_version,
