@@ -1,5 +1,6 @@
+import math
 from math import radians, cos, sin, asin, sqrt
-from typing import Union
+from typing import Union, List
 
 import numpy as np
 import scipy.interpolate as si
@@ -118,6 +119,89 @@ def project_point_onto_line(line: Union[np.ndarray, list], point: Union[np.ndarr
     return p, d
 
 
+def boxes_to_edges(boxes):
+    """
+    Source: https://stackoverflow.com/questions/4842613/merge-lists-that-share-common-elements
+
+        treat `l` as a Graph and returns it's edges
+        to_edges(['a','b','c','d']) -> [(a,b), (b,c),(c,d)]
+    """
+    it = iter(boxes)
+    last = next(it)
+
+    for current in it:
+        yield last, current
+        last = current
+
+
+def overlap(b1: List[float], b2: List[float], thres: float = .8) -> bool:
+    """ Check whether to bounding boxes have an overlap larger than thres
+        Each box must be of form x_min, y_min, x_max, y_max
+
+    Parameters
+    ----------
+    thres
+    b1
+    b2
+    """
+    a1 = abs(b1[2] - b1[0]) * abs(b1[3] - b1[1])
+    a2 = abs(b2[2] - b2[0]) * abs(b2[3] - b2[1])
+
+    if b2[0] > b1[2] or b2[1] > b1[3]:  # Boxes not overlapping
+        return False
+    else:
+        inter = (b1[2] - b2[0]) * (b1[3] - b2[1])
+
+        r1 = inter / a1 if a1 > 0 else -1.0
+        r2 = inter / a2 if a2 > 0 else -1.0
+
+        if r1 > thres or r2 > thres:
+            return True
+        else:
+            return False
+
+
+def iou(b1: List[float], b2: List[float]):
+    """ Calculates the Intersection over Union metric for two bounding boxes
+        Each box must be of form x_min, y_min, x_max, y_max
+
+    Parameters
+    ----------
+    b1
+    b2
+    """
+    b1_x0, b1_y0 = b1[0], b1[1]
+    b1_x1, b1_y1 = b1[2], b1[3]
+
+    b2_x0, b2_y0 = b2[0], b2[1]
+    b2_x1, b2_y1 = b2[2], b2[3]
+
+    b1_w = b1[2] - b1[0]
+    b1_h = b1[3] - b1[1]
+
+    b2_w = b2[2] - b2[0]
+    b2_h = b2[3] - b2[1]
+
+    a1 = abs(b1_w) * abs(b1_h)
+    a2 = abs(b2_w) * abs(b2_h)
+
+    x0 = max(min(b1_x0, b1_x1), min(b2_x0, b2_x1))
+    y0 = max(min(b1_y0, b1_y1), min(b2_y0, b2_y1))
+    x1 = min(max(b1_x0, b1_x1), max(b2_x0, b2_x1))
+    y1 = min(max(b1_y0, b1_y1), max(b2_y0, b2_y1))
+
+    if x1 < x0 or y1 < y0:
+        return 0.0
+
+    inter = (x1 - x0) * (y1 - y0)
+    union = a1 + a2 - inter
+
+    iou = inter / union if union > 0 else -1.0
+
+    assert 0.0 <= iou <= 1.0
+    return iou
+
+
 def is_point_within_line_projection(line: Union[np.ndarray, list], point: Union[np.ndarray, list]) -> bool:
     """ Checks whether a given points line projection falls within the points that the define the line
 
